using System;
using System.Collections.Generic;
using System.Text;
using OtterUnitTestFramework.Public.Test;
using System.Reflection;

namespace OtterUnitTestFramework.DataStruct
{
    /// <summary>
    /// Enumerator responsible for setting the Approval status.
    /// </summary>
    internal enum TestApprovalStatus
    {
        HasPassed,
        HasFailed,
        None
    }
    
    /// <summary>
    /// String holding constants to be printed depending on the Test
    /// result.
    /// </summary>
    internal struct TestApprovalStatusToBePrinted
    {
        public const String HAS_PASSED = "PASSED";
        public const String HAS_FAILED = "FAILED";
        public const String NONE = "NOT TESTED";
    }

    internal struct TestCaseTemplateHeaderDisplayer
    {
        public const String EXECUTE_TEST = "Execute Test";
        public const String CLASS_NAME = "Class";
        public const String METHOD_NAME = "Method";
        public const String HAS_PASSED = "Test status";
    }
    /// <summary>
    /// Hold information for displaying ane executing Test Case Templates.
    /// </summary>
    internal class TestCaseTemplateDisplayer
    {
        private bool _executeTest;
        private TestApprovalStatus _hasPassed;
        private MethodInfo _method;
        private TestCaseTemplate _testCaseTemplate;
        private String _errorMessage;

        /// <summary>
        /// Constructor - populate the whole object with thie method's parameters.
        /// </summary>
        /// <param name="testCaseTemplate">Test Case Template</param>
        /// <param name="method">Method to be executed</param>
        public TestCaseTemplateDisplayer(TestCaseTemplate testCaseTemplate, MethodInfo method)
        {
            _testCaseTemplate = testCaseTemplate;
            _method = method;
            _hasPassed = TestApprovalStatus.None;
            _executeTest = true;
        }

        /// <summary>
        /// Gets or Sets an Error Message
        /// </summary>
        internal String ErrorMessage
        {
            get { return _errorMessage; }
            set { _errorMessage = value; }
        }

        /// <summary>
        /// Gets the value of the status to be printed according
        /// to the status set in HasPassed.
        /// </summary>
        public String HasPassedToBePrinted
        {
            get
            {
                String returnValue = String.Empty;
                switch (_hasPassed)
                {
                    case TestApprovalStatus.HasFailed:
                        returnValue = TestApprovalStatusToBePrinted.HAS_FAILED;
                        break;
                    case TestApprovalStatus.HasPassed:
                        returnValue = TestApprovalStatusToBePrinted.HAS_PASSED;
                        break;
                    case TestApprovalStatus.None:
                        returnValue = TestApprovalStatusToBePrinted.NONE;
                        break;
                }
                //return value
                return returnValue;
            }
        }

        /// <summary>
        /// Gets or Sets the Method
        /// </summary>
        internal MethodInfo Method
        {
            get { return _method; }
            set { _method = value; }
        }

        /// <summary>
        /// Gets or Sets the Test Case Template
        /// </summary>
        internal TestCaseTemplate TestCaseTemplate
        {
            get { return _testCaseTemplate; }
            set { _testCaseTemplate = value; }
        }

        /// <summary>
        /// Gets or Sets value for Test Approval Status
        /// </summary>
        internal TestApprovalStatus HasPassed
        {
            get { return _hasPassed; }
            set { _hasPassed = value; }
        }

        /// <summary>
        /// Gets or Sets value for whether or not to execute the test
        /// </summary>
        public bool ExecuteTest
        {
            get { return _executeTest; }
            set { _executeTest = value; }
        }

        /// <summary>
        /// Gets or Sets value for Class Name
        /// </summary>
        public String ClassName
        {
            get { return _testCaseTemplate.GetType().Name; }
        }

        /// <summary>
        /// Gets the value for Method Name
        /// </summary>
        public String MethodName
        {
            get { return _method.Name; }
        }
    }
}
