using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using OtterUnitTestFramework.Exceptions;
using OtterUnitTestFramework.Forms;

namespace OtterUnitTestFramework.Public
{
    /// <summary>
    /// This class makes assertions regiarding the Unit Tests. They
    /// should be used in the implemented Test Case Template implementations.
    /// </summary>
    public class Assert
    {
        #region Assert.IsTrue

        /// <summary>
        /// Do nothing in case the expression is indeed true or fail the
        /// Test in case the value is false.
        /// </summary>
        /// <param name="expression">Expression to be evaluated</param>
        /// <param name="erroMessage">Error message to be displayed in case the test fails</param>
        public static void IsTrue(bool expression, String errorMessage)
        {
            //in case the expression is not true, throw an exception, otherwise do nothing
            if (!expression)
            {
                throw new AssertionException(FormatMessage("The Assert.IsTrue(<parameters>) has failed. The evaluated expression is false.", errorMessage));
            }
        }

        /// <summary>
        /// Do nothing in case the expression is indeed true or fail the
        /// Test in case the value is false.
        /// </summary>
        /// <param name="expression">Expression to be evaluated</param>
        public static void IsTrue(bool expression)
        {
            //call overloaded method
            IsTrue(expression, String.Empty);
        }

        #endregion

        #region Assert.IsFalse

        /// <summary>
        /// Do nothing in case the expression is indeed false or fail the
        /// test if the value (evaluated expression) is true.
        /// </summary>
        /// <param name="expression">Expression to be evaluated</param>
        /// <param name="errorMessage">Error message to be displayed in case the test fails</param>
        public static void IsFalse(bool expression, String errorMessage)
        {
            //in the the expression is true, throw an exception, otherwise do nothing
            if (expression)
            {
                throw new AssertionException(FormatMessage("The Assert.IsFalse(<parameters>) has failed. The evaluated expression is true.", errorMessage));
            }
        }

        /// <summary>
        /// Do nothing in case the expression is indeed false or fail the
        /// test if the value (evaluated expression) is true.
        /// </summary>
        /// <param name="expression">Expression to be evaluated</param>
        public static void IsFalse(bool expression)
        {
            //call overloaded methods
            IsFalse(expression, String.Empty);
        }

        #endregion

        #region Assert.AreEqual

        /// <summary>
        /// Do nothing in case the expressions are equal or fail the
        /// Test in case they are not.
        /// </summary>
        /// <param name="firstElement">Element to be compared to its counterpart - secondElement</param>
        /// <param name="secondElement">Element to be compared to its counterpart - firstElement</param>
        /// <param name="errorMessage">Error message to be displayed in case the test fails</param>
        public static void AreEqual(Object firstElement, Object secondElement, String errorMessage)
        {
            //only proceed in case neither of the elements are null
            if (firstElement == null)
            {
                throw new AssertionException(FormatMessage("The first element entered as the Assert.AreEqual(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            else if (secondElement == null)
            {
                throw new AssertionException(FormatMessage("The second element entered as the Assert.AreEqual(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            //in the elements are not equal throw an exception, otherwise do nothing
            else if (!firstElement.Equals(secondElement))
            {
                throw new AssertionException(FormatMessage("The Assert.AreEqual(<parameters>) has failed. The compared elements are different.", errorMessage));
            }
        }

        /// <summary>
        /// Do nothing in case the expressions are equal or fail the
        /// Test in case they are not.
        /// </summary>
        /// <param name="firstElement">Element to be compared to its counterpart - secondElement</param>
        /// <param name="secondElement">Element to be compared to its counterpart - firstElement</param>
        public static void AreEqual(Object firstElement, Object secondElement)
        {
            //call overloaded method
            AreEqual(firstElement, secondElement, String.Empty);
        }

        #endregion

        #region Assert.AreNotEqual

        /// <summary>
        /// Do nothing in case the expressions are not equal or fail the
        /// Test in case they are not.
        /// </summary>
        /// <param name="firstElement">Element to be compared to its counterpart - secondElement</param>
        /// <param name="secondElement">Element to be compared to its counterpart - firstElement</param>
        /// <param name="errorMessage">Error message to be displayed in case the test fails</param>
        public static void AreNotEqual(Object firstElement, Object secondElement, String errorMessage)
        {
            //only proceed in case neither of the elements are null
            if (firstElement == null)
            {
                throw new AssertionException(FormatMessage("The first element entered as the Assert.AreNotEqual(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            else if (secondElement == null)
            {
                throw new AssertionException(FormatMessage("The second element entered as the Assert.AreNotEqual(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            //in the elements are equal throw an exception, otherwise do nothing
            else if (firstElement.Equals(secondElement))
            {
                throw new AssertionException(FormatMessage("The Assert.AreNotEqual(<parameters>) has failed. The compared elements are equal.", errorMessage));
            }
        }

        /// <summary>
        /// Do nothing in case the expressions are not equal or fail the
        /// Test in case they are not.
        /// </summary>
        /// <param name="firstElement">Element to be compared to its counterpart - secondElement</param>
        /// <param name="secondElement">Element to be compared to its counterpart - firstElement</param>
        public static void AreNotEqual(Object firstElement, Object secondElement)
        {
            //call overloaded method
            AreNotEqual(firstElement, secondElement, String.Empty);
        }

        #endregion

        #region Assert.AreReferencesEqual

        /// <summary>
        /// Compare two objects (first and second method's parameters) and in case
        /// they refer to the same object, the Test passes, otherwise it fails.
        /// </summary>
        /// <param name="firstElement">First object to have its reference compared to the second one.</param>
        /// <param name="secondElement">Second object to have tis reference compared to the first one.</param>
        /// <param name="errorMessage">Error message to be displayed in case the test fails</param>
        public static void AreReferencesEqual(Object firstElement, Object secondElement, String errorMessage)
        {
            //only proceed in case neither of the elements are null
            if (firstElement == null)
            {
                throw new AssertionException(FormatMessage("The first element entered as the Assert.AreReferencesEqual(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            else if (secondElement == null)
            {
                throw new AssertionException(FormatMessage("The second element entered as the Assert.AreReferencesEqual(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            //the compared object must be of the same type
            else if (!firstElement.GetType().ToString().Equals(secondElement.GetType().ToString()))
            {
                throw new AssertionException(FormatMessage("The first and second element entered as the Assert.AreReferencesEqual(<parameters>) method's parameter are of different types thus the evaluation has failed", errorMessage));
            }
            //in case the objects are not from the same reference, throw an exception
            else if (!Object.ReferenceEquals(firstElement, secondElement))
            {
                throw new AssertionException(FormatMessage("The Assert.AreReferencesEqual(<parameters>) has failed. The compared elements are not from the same reference.", errorMessage));
            }
        }

        /// <summary>
        /// Compare two objects (first and second method's parameters) and in case
        /// they refer to the same object, the Test passes, otherwise it fails.
        /// </summary>
        /// <param name="firstElement">First object to have its reference compared to the second one.</param>
        /// <param name="secondElement">Second object to have tis reference compared to the first one.</param>
        public static void AreReferencesEqual(Object firstElement, Object secondElement)
        {
            //call overloaded method
            AreReferencesEqual(firstElement, secondElement, String.Empty);
        }

        #endregion

        #region Assert.AreReferencesNotEqual

        /// <summary>
        /// Compare two objects (first and second method's parameters) and in case
        /// they do not refer to the same object, the Test passes, otherwise it fails.
        /// </summary>
        /// <param name="firstElement">First object to have its reference compared to the second one.</param>
        /// <param name="secondElement">Second object to have tis reference compared to the first one.</param>
        /// <param name="errorMessage">Error message to be displayed in case the test fails</param>
        public static void AreReferencesNotEqual(Object firstElement, Object secondElement, String errorMessage)
        {
            //only proceed in case neither of the elements are null
            if (firstElement == null)
            {
                throw new AssertionException(FormatMessage("The first element entered as the Assert.AreReferencesEqual(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            else if (secondElement == null)
            {
                throw new AssertionException(FormatMessage("The second element entered as the Assert.AreReferencesEqual(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            //in case the objects are from the same reference, throw an exception
            else if (Object.ReferenceEquals(firstElement, secondElement))
            {
                throw new AssertionException(FormatMessage("The Assert.AreReferencesNotEqual(<parameters>) has failed. The compared elements are from the same reference.", errorMessage));
            }
        }

        /// <summary>
        /// Compare two objects (first and second method's parameters) and in case
        /// they do not refer to the same object, the Test passes, otherwise it fails.
        /// </summary>
        /// <param name="firstElement">First object to have its reference compared to the second one.</param>
        /// <param name="secondElement">Second object to have tis reference compared to the first one.</param>
        public static void AreReferencesNotEqual(Object firstElement, Object secondElement)
        {
            //call overloaded method
            AreReferencesNotEqual(firstElement, secondElement, String.Empty);
        }

        #endregion

        #region Assert.IsGreater

        /// <summary>
        /// Compare expectedGreater and expectedLess objects and, in case
        /// expectedGreater is greater than expectedLess, the Test succeeds,
        /// otherwise it fails.
        /// </summary>
        /// <param name="expectedGreater">Intance to be compared expected to be greater.</param>
        /// <param name="expectedLess">Instance to be campared expected to bel less.</param>
        /// <param name="errorMessage">Error message to be displayed in case the test fails.</param>
        public static void IsGreater(IComparable expectedGreater, IComparable expectedLess, String errorMessage)
        {
            //only proceed in case neither of the elements are null
            if (expectedGreater == null)
            {
                throw new AssertionException(FormatMessage("The first element (expectedGreater) entered as the Assert.IsGreater(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            else if (expectedLess == null)
            {
                throw new AssertionException(FormatMessage("The second element (expectedLess) entered as the Assert.IsGreater(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            //in case the expecteGreater is not higher than the expectedLess, throw an exception
            else if (!(expectedGreater.CompareTo(expectedLess) > 0))
            {
                throw new AssertionException(FormatMessage("The Assert.IsGreater(<parameters>) has failed. The expectedGreater is not greater than the expectedLess.", errorMessage));
            }
        }

        /// <summary>
        /// Compare expectedGreater and expectedLess objects and, in case
        /// expectedGreater is greater than expectedLess, the Test succeeds,
        /// otherwise it fails.
        /// </summary>
        /// <param name="expectedGreater">Intance to be compared expected to be greater.</param>
        /// <param name="expectedLess">Instance to be campared expected to bel less.</param>
        public void IsGreater(IComparable expectedGreater, IComparable expectedLess)
        {
            //call overloaded method
            IsGreater(expectedGreater, expectedLess, String.Empty);
        }

        #endregion

        #region Assert.IsLess

        /// <summary>
        /// Compare expectedLess and expectedGreater objects and, in case
        /// expectedLess is less than expectedGreat, the Test succeeds,
        /// otherwise it fails.
        /// </summary>
        /// <param name="expectedLess">Instance to be campared expected to bel less.</param>
        /// <param name="expectedGreater">Intance to be compared expected to be greater.</param>
        /// <param name="errorMessage">Error message to be displayed in case the test fails.</param>
        public static void IsLess(IComparable expectedLess, IComparable expectedGreater, String errorMessage)
        {
            //only proceed in case neither of the elements are null
            if (expectedLess == null)
            {
                throw new AssertionException(FormatMessage("The first element (expectedLess) entered as the Assert.IsLess(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            else if (expectedGreater == null)
            {
                throw new AssertionException(FormatMessage("The second element (expectedGreater) entered as the Assert.IsLess(<parameters>) method's parameter is null thus the evaluation has failed", errorMessage));
            }
            //in case the expecteLess is not less than the expectedGreater, throw an exception
            else if (!(expectedLess.CompareTo(expectedGreater) < 0))
            {
                throw new AssertionException(FormatMessage("The Assert.IsLess(<parameters>) has failed. The expectedLess is not less than the expectedGreater.", errorMessage));
            }
        }

        /// <summary>
        /// Compare expectedLess and expectedGreater objects and, in case
        /// expectedLess is less than expectedGreat, the Test succeeds,
        /// otherwise it fails.
        /// </summary>
        /// <param name="expectedLess">Instance to be campared expected to bel less.</param>
        /// <param name="expectedGreater">Intance to be compared expected to be greater.</param>
        public static void IsLess(IComparable expectedLess, IComparable expectedGreater)
        {
            //call overloade method
            IsLess(expectedLess, expectedGreater, String.Empty);
        }

        #endregion

        #region Assert.IsNotNull

        /// <summary>
        /// Do nothing in case the expression is indeed not null or fail the
        /// Test in case the value is null.
        /// </summary>
        /// <param name="expression">Expression to be evaluated</param>
        /// <param name="errorMessage">Error message to be displayed in case the test fails</param>
        public static void IsNotNull(Object expression, String errorMessage)
        {
            //in case the expression null, throw an exception, otherwise do nothing
            if (expression == null)
            {
                throw new AssertionException(FormatMessage("The Assert.IsNotNull(<parameters>) has failed. The evaluated expression is null.", errorMessage));
            }
        }

        /// <summary>
        /// Do nothing in case the expression is indeed not null or fail the
        /// Test in case the value is null.
        /// </summary>
        /// <param name="expression">Expression to be evaluated</param>
        public static void IsNotNull(Object expression)
        {
            //call overloaded method
            IsNotNull(expression, String.Empty);
        }

        #endregion

        #region Assert.IsNull

        /// <summary>
        /// Do nothing in case the expression is indeed null or fail the
        /// Test in case the value is not null.
        /// </summary>
        /// <param name="expression">Expression to be evaluated</param>
        /// <param name="errorMessage">Error message to be displayed in case the test fails</param>
        public static void IsNull(Object expression, String errorMessage)
        {
            //in case the expression is not null, throw an exception, otherwise do nothing
            if (expression != null)
            {
                throw new AssertionException(FormatMessage("The Assert.IsNotNull(<parameters>) has failed. The evaluated expression is not null.", errorMessage));
            }
        }

        /// <summary>
        /// Do nothing in case the expression is indeed null or fail the
        /// Test in case the value is not null.
        /// </summary>
        /// <param name="expression">Expression to be evaluated</param>
        public static void IsNull(Object expression)
        {
            //call overloaded method
            IsNull(expression, String.Empty);
        }

        #endregion

        #region Assert.Fail

        /// <summary>
        /// States a failure in the test stating the error message.
        /// </summary>
        /// <param name="message">Error Message</param>
        public static void Fail(String message)
        {
            throw new AssertionException("The test has failed. Error: " + message); 
        }

        /// <summary>
        /// States a failure in the test printing information entered
        /// within the excepiton entered as this method's parameter.<br/>
        /// This method ought to be called when an exception is caught in
        /// a Test Case Template implementation.
        /// </summary>
        /// <param name="exception">Exception caught in a Test Case Template Implementation.</param>
        public static void Fail(Exception exception)
        {
            /*
             * In case there is a base exception, display the message regarding 
             * it, otherwise display info of this exception. Obviously, start
             * only in case the exception is not null
             */
            if (exception == null)
            {
                throw new NullReferenceException("There was a null exception entered in the method Assert.Fail(Exception)");
            }
            else if (exception.GetBaseException() != null)
            {
                throw new AssertionException("The test has failed.", exception.GetBaseException());
            }
            else
            {
                Fail(exception.Message);
            }
        }

        #endregion

        #region Assert.ShowMessage

        /// <summary>
        /// Show a message (Dialog Box) entered as this message parameter.
        /// </summary>
        /// <param name="message">Message to be displayed</param>
        public static void ShowMessage(String message)
        {
            SimpleMessageForm messageForm = new SimpleMessageForm(message);
            messageForm.ShowDialog();
        }

        #endregion

        #region Private methods

        /// <summary>
        /// Formats a message to be displayed as the following format: providedMessage - coreMessage.
        /// Verifications regarding empty and null messages are made.
        /// </summary>
        /// <param name="coreMessage">Core message. Goes at the end. It must is never be null nor empty.</param>
        /// <param name="providedMessage">Provided message. Goes at the beginning.</param>
        /// <returns>Formated message as the following: providedMessage - coreMesage. In case provided message is null or empty, only the coreMessage is returned.</returns>
        private static String FormatMessage(String coreMessage, String providedMessage)
        {
            String CONCAT_STRING = " - ";
            //message te be returned
            String formattedMessage = String.Empty;
            //concat messages in case the provided message has content
            if (providedMessage != null && !providedMessage.Equals(String.Empty))
            {
                formattedMessage = providedMessage + CONCAT_STRING + coreMessage;
            }
            else
            {
                //return only the core message
                formattedMessage = coreMessage;
            }
            //return the formatted message
            return formattedMessage;
        }

        #endregion
    }
}
