using System;
using System.Collections.Generic;
using System.Text;

namespace OtterUnitTestFramework.Public
{
    /// <summary>
    /// Expected Exception Attribute. This attribute is to mark a method
    /// for stating a certain exception is expected to be thrown.
    /// </summary>
    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false)]
    public sealed class ExpectedExceptionAttribute : Attribute
    {
        private Type[] expectedExceptions;

        /// <summary>
        /// Constructor for a given type of exception
        /// </summary>
        /// <param name="exceptionType">The type of the expected exception</param>
        public ExpectedExceptionAttribute(Type exceptionType)
        {
            this.expectedExceptions = new Type[1]{exceptionType};
            
        }

        /// <summary>
        /// Constructor for a set of given types of exception
        /// </summary>
        /// <param name="exceptionType">The type of the expected exception array</param>
        public ExpectedExceptionAttribute(Type[] exceptionTypes)
        {
            this.expectedExceptions = exceptionTypes;
        }

        /// <summary>
        /// Gets or sets the expected exception type
        /// </summary>
        public Type[] ExceptionTypes
        {
            get { return expectedExceptions; }
            set { expectedExceptions = value; }
        }
    }
}
