using System;
using System.Collections.Generic;
using System.Text;

namespace OtterUnitTestFramework.Public.Test
{
    /// <summary>
    /// Struct which hold constants that are used by the OtterUnitTestFramework.Test.TestCaseTemplate class.
    /// </summary>
    public struct TestCaseTemplateConstants
    {
        /// <summary>
        /// The constant part that must be used in the beginning of 
        /// each test case method.
        /// </summary>
        public const String TEST_METHOD_INITIAL_NAME = "TestCase";
    }
    /// <summary>
    /// This class is the skeleton for the tests. It means all
    /// tests that will be generated, must inherit from this class.
    /// In order to generate a test, methods should be created with
    /// the following name pattern: testCase* - it means, they should start
    /// with "testCase" and than complementing it with relevant information.
    /// Note that this clas has two methods that can be overriden: StartUp() and
    /// TearDown(). The former is called before every test method is run and the
    /// latter is called after every test method is run.
    /// </summary>
    public abstract class TestCaseTemplate : MarshalByRefObject
    {
        /// <summary>
        /// This method is called before every method of this class is
        /// run (as a test). On should override it in order to add behavior to it.
        /// </summary>
        public virtual void StartUp()
        {
            //do nothing
        }

        /// <summary>
        /// This method is called after every method of this class is
        /// run (as a test). One should override it in order to add behavior to it.
        /// </summary>
        public virtual void TearDown()
        {
            //do nothing
        }
    }
}
