using System;
using System.Collections.Generic;
using System.Text;
using OtterUnitTestFramework.Public;
using OtterUnitTestFramework.Public.Test;
using OtterUnitTestFrameworkRunner.Classes;
using OtterUnitTestFrameworkRunner.Exceptions;

namespace OtterUnitTestFrameworkRunner.Test
{
    /// <summary>
    /// Implement the Test Case Template to run some dumny tests.
    /// </summary>
    public class TestCase : TestCaseTemplate
    {
        /// <summary>
        /// Just a success test that does nothing.
        /// </summary>
        public void TestCaseTestSuccess()
        {
            //do nothing
        }

        /// <summary>
        /// Just a test that will fail by making an assertion.
        /// </summary>
        public void TestCaseFail()
        {
            //make false assertion
            Assert.Fail("This is is doomed to fail!");
        }

        /// <summary>
        /// Simply throw an exception (do not catch it). 
        /// </summary>
        public void TestCaseThrowExceptionAndDoNotCatchItInOrderToFail()
        {
            Executer executer = new Executer();
            //throw an exception
            executer.ThrowException();
        }

        /// <summary>
        /// Test the usage of the ExcpectedException tag.
        /// </summary>
        [ExpectedException(new Type[] { typeof(ApplicationException), typeof(TestException) })]
        public void TestCaseThrowExceptionAndUseTheExpectedExceptionTag()
        {
            Executer executer = new Executer();
            executer.ThrowTestException();
        }

        /// <summary>
        /// Throw an exception and catch it in order to fail it in the catch block.
        /// </summary>
        public void TestCaseThrowExceptionAndCatchItInOrderToFailInInTheCatchBlock()
        {
            try
            {
                //throw an exception
                TestCaseThrowExceptionAndDoNotCatchItInOrderToFail();
            }
            catch (Exception ex)
            {
                Assert.Fail(ex);
            }
        }

        /// <summary>
        /// Execute comparision assertions with null elements.
        /// </summary>
        public void TestCaseByEnteringANullElementInComparisons()
        {
            bool hasFailed = false;
            String errorMessage = String.Empty;
            try
            {
                Assert.AreEqual(new Object(), null);
                hasFailed = true;
                errorMessage = "This test should have failed due to a null element in the comparison assertion";
            }
            catch (Exception ex)
            {
                //success
            }

            try
            {
                Assert.AreEqual(null, new Object());
                hasFailed = true;
                errorMessage = "This test should have failed due to a null element in the comparison assertion";
            }
            catch (Exception ex)
            {
                //success
            }

            try
            {
                Assert.AreEqual(null, null);
                hasFailed = true;
                errorMessage = "This test should have failed due to a null element in the comparison assertion";
            }
            catch (Exception ex)
            {
                //success
            }

            try
            {
                Assert.AreNotEqual(new Object(), null);
                hasFailed = true;
                errorMessage = "This test should have failed due to a null element in the comparison assertion";
            }
            catch (Exception ex)
            {
                //success
            }

            try
            {
                Assert.AreNotEqual(null, new Object());
                hasFailed = true;
                errorMessage = "This test should have failed due to a null element in the comparison assertion";
            }
            catch (Exception ex)
            {
                //success
            }

            try
            {
                Assert.AreNotEqual(null, null);
                hasFailed = true;
                errorMessage = "This test should have failed due to a null element in the comparison assertion";
            }
            catch (Exception ex)
            {
                //success
            }

            //in case there was a failure, throw an exception
            if (hasFailed)
            {
                throw new Exception(errorMessage);
            }
        }

        /// <summary>
        /// This method should not be included in the list test
        /// because it has a return type different than void.
        /// </summary>
        /// <returns></returns>
        public Int32 TestCaseWrongMethodReturnType()
        {
            //do nothing
            return 0;
        }

        /// <summary>
        /// This method should not be included because it has parameters
        /// when it ought not.
        /// </summary>
        /// <param name="param1">Dumny Value</param>
        /// <param name="param2">Dumny Value</param>
        public void TestCaseWrongMethodSignature(Int32 param1, Double param2)
        {
            //do nothing
        }

        /// <summary>
        /// This method should not be included in the test because it
        /// has the wrong return type (it is not void) and it has parameters.
        /// </summary>
        /// <param name="param1">Dumny Value</param>
        /// <param name="param2">Dumny Value</param>
        /// <returns>Dumny Value</returns>
        public Double TestCaseWrongMethodSignatureAndReturnType(Int32 param1, String param2)
        {
            return Double.MaxValue;
            //do nothing
        }

        /// <summary>
        /// Teste the retrieval n App.Config element.
        /// </summary>
        public void TestCaseRetrieveAppConfigElement()
        {
            String VALUE = "MySuperValue";
            //get element
            String retrievedElement = System.Configuration.ConfigurationManager.AppSettings["TheValue"];
            //print assembly
            Assert.ShowMessage(AppDomain.CurrentDomain.FriendlyName + " base dicrectory: " + AppDomain.CurrentDomain.BaseDirectory);
            //make assertion
            Assert.AreEqual(retrievedElement, VALUE);
        }
    }
}
